use maud::{html, PreEscaped};

use crate::templates;

// /scuttlebutt/peers
pub fn peers() -> PreEscaped<String> {
    let back = "/".to_string();
    let title = "Scuttlebutt Peers".to_string();

    let content = html! {
        (PreEscaped("<!-- SCUTTLEBUTT PEERS -->"))
        div class="card center" {
            div class="card-container" {
                (PreEscaped("<!-- BUTTONS -->"))
                div id="buttons" {
                    a id="friends" class="button button-primary center" href="/scuttlebutt/friends" title="List Friends" { "Friends" }
                    a id="follows" class="button button-primary center" href="/scuttlebutt/follows" title="List Follows" { "Follows" }
                    a id="followers" class="button button-primary center" href="/scuttlebutt/followers" title="List Followers" { "Followers" }
                    a id="blocks" class="button button-primary center" href="/scuttlebutt/blocks" title="List Blocks" { "Blocks" }
                }
            }
        }
    };

    templates::base::base(back, title, content)
}

// /scuttlebutt/friends
// /scuttlebutt/follows
// /scuttlebutt/followers
// /scuttlebutt/blocks
pub fn peers_list(title: String) -> PreEscaped<String> {
    let back = "/scuttlebutt/peers".to_string();

    let content = html! {
        (PreEscaped("<!-- SCUTTLEBUTT PEERS LIST -->"))
        div class="card center" {
            ul class="list" {
                // for peer in peers
                li {
                    a class="list-item link light-bg" href="/scuttlebutt/profile/(pub_key)" {
                        img id="peerImage" class="icon list-icon" src="{ image_path }" alt="{ peer_name }'s profile image";
                        p id="peerName" class="list-text" { "(name)" }
                        label class="label-small label-ellipsis list-label font-gray" for="peerName" title="{ peer_name }'s Public Key" { "(public_key)" }
                    }
                }
            }
        }
    };

    templates::base::base(back, title, content)
}

// /scuttlebutt/private
pub fn private() -> PreEscaped<String> {
    let back = "/".to_string();
    let title = "Private Messages".to_string();

    let content = html! {
        div class="card center" {
            div class="card-container" {
                p { "private message content goes here" }
            }
        }
    };

    templates::base::base(back, title, content)
}

// /scuttlebutt/profile
pub fn profile(flash_msg: Option<String>, flash_name: Option<String>) -> PreEscaped<String> {
    let back = "/".to_string();
    let title = "Scuttlebutt Profile".to_string();

    let content = html! {
        (PreEscaped("<!-- USER PROFILE -->"))
        div class="card center" {
            (PreEscaped("<!-- PROFILE INFO BOX -->"))
            div class="capsule capsule-profile" title="Scuttlebutt account profile information" {
                (PreEscaped("<!-- edit profile button -->"))
                img id="editProfile" class="icon-small nav-icon-right" src="/icons/pencil.svg" alt="Profile picture";
                (PreEscaped("<!-- PROFILE BIO -->"))
                (PreEscaped("<!-- profile picture -->"))
                img id="profilePicture" class="icon-large" src="{ image_path }" alt="Profile picture";
                (PreEscaped("<!-- name, public key & description -->"))
                p id="profileName" class="card-text" title="Name" { "(name)" }
                label class="label-small label-ellipsis font-gray" style="user-select: all;" for="profileName" title="Public Key" { "(public_key)" }
                p id="profileDescription" style="margin-top: 1rem" class="card-text" title="Description" { "(description)" }
            }
            (PreEscaped("<!-- PUBLIC POST FORM -->"))
            form id="postForm" action="/scuttlebutt/post" method="post" {
                (PreEscaped("<!-- input for message contents -->"))
                textarea id="publicPost" class="center input message-input" title="Compose Public Post" { }
                input id="publishPost" class="button button-primary center" title="Publish" type="submit" value="Publish";
            }
            (PreEscaped("<!-- BUTTONS -->"))
            (PreEscaped("<!-- TODO: each of these buttons needs to be a form with a public key -->"))
            div id="buttons" {
                a id="followPeer" class="button button-primary center" href="/scuttlebutt/follow" title="Follow Peer" { "Follow" }
                a id="blockPeer" class="button button-warning center" href="/scuttlebutt/block" title="Block Peer" { "Block" }
                a id="privateMessage" class="button button-primary center" href="/scuttlebutt/private_message" title="Private Message" { "Private Message" }
            }
            (PreEscaped("<!-- FLASH MESSAGE -->"))
            (templates::snippets::flash_message(flash_msg, flash_name))
        }
    };

    templates::base::base(back, title, content)
}
