use maud::{html, PreEscaped};

use crate::context::admin;
use crate::templates;

// /settings/admin
pub fn menu() -> PreEscaped<String> {
    let back = "/settings".to_string();
    let title = "Administrator Settings".to_string();

    let content = html! {
        (PreEscaped("<!-- ADMIN SETTINGS MENU -->"))
        div class="card center" {
            (PreEscaped("<!-- BUTTONS -->"))
            div id="settingsButtons" {
                a id="change" class="button button-primary center" href="/settings/admin/change_password" title="Change Password" { "Change Password" }
                a id="configure" class="button button-primary center" href="/settings/admin/configure" title="Configure Admin" { "Configure Admin" }
            }
        }
    };

    templates::base::base(back, title, content)
}

// /settings/admin/add
pub fn add(flash_msg: Option<String>, flash_name: Option<String>) -> PreEscaped<String> {
    let back = "/settings/admin/configure".to_string();
    let title = "Add Administrator".to_string();

    let content = html! {
        (PreEscaped("<!-- ADD ADMIN FORM -->"))
        div class="card center" {
            div class="card-container" {
                form id="addAdminForm" action="/settings/admin/add" method="post" {
                    input id="ssb_id" name="ssb_id" class="center input" type="text" placeholder="SSB ID" title="SSB ID of Admin" value="";
                    div id="buttonDiv" {
                        input id="addAdmin" class="button button-primary center" title="Add" type="submit" value="Add";
                        a class="button button-secondary center" href="/settings/admin/configure" title="Cancel" { "Cancel" }
                    }
                }
                // render flash message, if any
                (templates::snippets::flash_message(flash_msg, flash_name))
            }
        }
    };

    templates::base::base(back, title, content)
}

// /settings/admin/configure
pub fn configure(flash_msg: Option<String>, flash_name: Option<String>) -> PreEscaped<String> {
    let ssb_admin_ids = admin::ssb_admin_ids();

    let back = "/settings/admin".to_string();
    let title = "Administrator Configuration".to_string();

    let content = html! {
        (PreEscaped("<!-- CONFIGURE ADMIN PAGE -->"))
        div class="card center" {
            div class="text-container" {
                h4 { "Current Admins" }
                @match ssb_admin_ids {
                    Ok(admins) => {
                        @if admins.is_empty() {
                            div { "No administators are currently configured" }
                        } else {
                            @for admin in admins {
                                div {
                                    form action="/settings/admin/delete" method="post" {
                                        input type="hidden" name="ssb_id" value="{{admin}}";
                                        input type="submit" value="X" title="Delete" {
                                            span { (admin) }
                                        }
                                    }
                                }
                            }
                        }
                    },
                    Err(e) => div { "Encountered an error while trying to retrieve list of administrators: " (e) }
                }
                a class="button button-primary center full-width" style="margin-top: 25px;" href="/settings/admin/add" title="Add Admin" { "Add Admin" }
            }
            // render flash message, if any
            (templates::snippets::flash_message(flash_msg, flash_name))
        }
    };

    templates::base::base(back, title, content)
}

// /settings/admin/change_password
pub fn change_password(
    flash_msg: Option<String>,
    flash_name: Option<String>,
) -> PreEscaped<String> {
    let back = "/settings/admin".to_string();
    let title = "Change Password".to_string();

    let content = html! {
        (PreEscaped("<!-- CHANGE PASSWORD FORM -->"))
        div class="card center" {
            div class="form-container" {
                form id="changePassword" action="/settings/admin/change_password" method="post" {
                    (PreEscaped("<!-- input for current password -->"))
                    input id="currentPassword" class="center input" name="current_password" type="password" placeholder="Current password" title="Current password" autofocus;
                    (PreEscaped("<!-- input for new password -->"))
                    input id="newPassword" class="center input" name="new_password1" type="password" placeholder="New password" title="New password";
                    (PreEscaped("<!-- input for duplicate new password -->"))
                    input id="newPasswordDuplicate" class="center input" name="new_password2" type="password" placeholder="Re-enter new password" title="New password duplicate";
                    div id="buttonDiv" {
                        input id="savePassword" class="button button-primary center" title="Add" type="submit" value="Save";
                        a class="button button-secondary center" href="/settings/admin" title="Cancel" { "Cancel" }
                    }
                }
                // render flash message, if any
                (templates::snippets::flash_message(flash_msg, flash_name))
            }
        }
    };

    templates::base::base(back, title, content)
}
